import os
import shutil
import tempfile
import subprocess

from django.test import TestCase as DjangoTestCase


class MigrationIntegrationTests(DjangoTestCase):
    """
    In order to test migrations in the test project, via the public migrations
    interface (the management commands), we copy the test directory into a
    temporary location, and run management commands against that project.
    """

    def setUp(self):
        self.tmp = tempfile.mkdtemp()
        src = os.path.dirname(__file__)

        shutil.copytree(
            src,
            os.path.join(self.tmp, os.path.basename(src)),
        )

    def tearDown(self):
        shutil.rmtree(self.tmp)

    def run_command(self, command, expect_failure=False):
        failed = False
        try:
            return subprocess.check_output(
                'python %s/tests/manage.py %s' % (self.tmp, command),
                shell=True,
            )
        except subprocess.CalledProcessError as e:
            if expect_failure:
                self.assertNotEqual(e.returncode, 0)
                failed = True
            else:
                print(e.output)
                self.assertEqual(e.returncode, 0)

        self.assertEqual(failed, expect_failure)

    def assertMigrationExists(self, migration_number, exists=True):
        migration_dir = os.path.join(self.tmp, 'tests', 'migrations')

        migration_numbers = [
            x.split('_')[0] for x in os.listdir(migration_dir)
        ]

        self.assertEqual(migration_number in migration_numbers, exists)

    def test_make_migrations(self):
        self.run_command('makemigrations tests')
        self.assertMigrationExists('0001')

    def test_migrate(self):
        self.run_command('makemigrations tests')
        self.assertMigrationExists('0001')
        self.run_command('migrate')

    def test_stable_migrations(self):
        self.run_command('makemigrations tests')
        self.assertMigrationExists('0001')

        self.run_command('migrate')

        self.run_command('makemigrations tests --exit', expect_failure=True)
        self.assertMigrationExists('0002', exists=False)



import unittest

import django
from django.db import models
from django.db.utils import IntegrityError
from django.core import serializers
from django.http import HttpRequest, Http404
from django.test import TestCase as DjangoTestCase
from django.template import RequestContext
from django.template.loader import render_to_string
from django.db.models.fields import NOT_PROVIDED

from django_enumfield import Enum, Item, get_enum_or_404
from django_enumfield.utils import TemplateErrorException

from .enums import TestModelEnum
from .models import TestModel, TestModelNull, TestModelRandomDefault


class ItemTests(unittest.TestCase):
    def test_item(self):
        item = Item(10, 'slug', "Display")

        self.assertEqual(item.value, 10)
        self.assertEqual(item.slug, 'slug')
        self.assertEqual(item.display, "Display")

    def test_invalid_types(self):
        with self.assertRaises(TypeError):
            Item('not an int', 'slug', "display")

        with self.assertRaises(TypeError):
            Item(10, 999, "display")

        with self.assertRaises(TypeError):
            Item(10, 'slug', 999)

    def test_str(self):
        self.assertEqual(str(Item(10, 'slug', "display")), 'slug')

    def test_repr(self):
        self.assertEqual(
            repr(Item(10, 'slug', "display")),
            "<enum.Item: 10 slug 'display'>",
        )

    def test_hash(self):
        self.assertEqual(hash(Item(10, 'slug', "display")), 10)

    def test_eq(self):
        item1 = Item(10, 'slug', "display")
        item2 = Item(10, 'slug', "display")
        item3 = Item(20, 'slug3', "display")

        self.assertEqual(item1, item2)
        self.assertNotEqual(item1, item3)

        self.assertEqual('slug', item1)
        self.assertEqual(item3, 'slug3')
        self.assertNotEqual(item2, 'slug2')

    def test_comparison(self):
        item1 = Item(10, 'slug1', "display1")
        item2 = Item(20, 'slug2', "display2")
        item2_copy = Item(20, 'slug2', "display2")

        self.assertLess(item1, item2)
        self.assertGreater(item2, item1)
        self.assertGreaterEqual(item2, item2_copy)
        self.assertLessEqual(item2, item2_copy)


class EnumConstructionTests(unittest.TestCase):
    def test_instance_based_enum(self):
        FooEnum = Enum(
            'FooEnum',
            Item(10, 'a', "Item A"),
            Item(20, 'b', "Item B"),
        )

        self.assertEqual(len(FooEnum), 2)
        self.assertEqual(FooEnum.A.slug, 'a')
        self.assertEqual(FooEnum.B.display, "Item B")
        self.assertEqual(FooEnum.from_value(10).slug, 'a')

    def test_dynamic_enum(self):
        FooEnum = Enum('FooEnum')
        FooEnum.add_item(Item(10, 'a', "Item A"))
        FooEnum.add_item(Item(20, 'b', "Item B"))

        self.assertEqual(len(FooEnum), 2)
        self.assertEqual(FooEnum.A.slug, 'a')
        self.assertEqual(FooEnum.B.display, "Item B")
        self.assertEqual(FooEnum.from_value(10).slug, 'a')

    def test_dynamic_enum_rejects_duplicate_value(self):
        FooEnum = Enum('FooEnum')
        FooEnum.add_item(Item(10, 'a', "Item A"))

        with self.assertRaises(ValueError):
            FooEnum.add_item(Item(10, 'b', "Item B"))

    def test_dynamic_enum_rejects_duplicate_slug(self):
        FooEnum = Enum('FooEnum')
        FooEnum.add_item(Item(10, 'a', "Item A"))

        with self.assertRaises(ValueError):
            FooEnum.add_item(Item(20, 'a', "Item B"))

    def test_simple_registry_enum(self):
        FooEnum = Enum('FooEnum')

        class A(Item):
            __enum__ = FooEnum

            value = 10
            display = "Item A"

        class B(Item):
            __enum__ = FooEnum

            value = 20
            display = "Item B"

        self.assertEqual(len(FooEnum), 2)
        self.assertEqual(FooEnum.A.slug, 'A')
        self.assertEqual(FooEnum.B.display, "Item B")
        self.assertEqual(FooEnum.from_value(10).slug, 'A')

    def test_registry_without_parent(self):
        FooEnum = Enum('FooEnum')

        class FooEnumItem(Item):
            __enum__ = FooEnum

            def display_extended(self):
                return "%s (%s)" % (self.display, self.value)

        class A(FooEnumItem):
            value = 10
            display = "Item A"

        class B(FooEnumItem):
            value = 20
            display = "Item B"

        self.assertEqual(len(FooEnum), 2)
        self.assertEqual(FooEnum.A.slug, 'A')
        self.assertEqual(FooEnum.B.display, "Item B")
        self.assertEqual(FooEnum.from_value(10).slug, 'A')

        self.assertEqual(FooEnum.A.display_extended(), "Item A (10)")


class EnumTests(unittest.TestCase):
    def setUp(self):
        super(EnumTests, self).setUp()

        FooEnum = Enum(
            'FooEnum',
            Item(10, 'a', "Item A"),
            Item(20, 'b', "Item B"),
        )

        LargeEnum = Enum(
            'LargeEnum',
            Item(10, 'item_a', "Item A"),
            Item(20, 'item_b', "Item B"),
            Item(30, 'item_c', "Item C"),
            Item(40, 'item_d', "Item D"),
            Item(50, 'item_e', "Item E"),
        )

        self.enum = FooEnum
        self.large_enum = LargeEnum

    def test_from_value(self):
        self.assertEqual(self.enum.from_value(10).slug, 'a')

        with self.assertRaises(ValueError):
            self.enum.from_value('a')

    def test_from_slug(self):
        self.assertEqual(self.enum.from_slug('b').value, 20)

        with self.assertRaises(TypeError):
            self.enum.from_slug(20)

        with self.assertRaises(ValueError) as cm:
            self.enum.from_slug('nope')

        self.assertIn(
            'nope',
            str(cm.exception),
            "Exception message should contain errenous slug",
        )

        self.assertIn(
            'a, b',
            str(cm.exception),
            "Exception message should contain valid slugs",
        )

        self.assertIn(
            self.enum.name,
            str(cm.exception),
            "Exception message should contain enum name",
        )

        with self.assertRaises(ValueError) as cm:
            self.large_enum.from_slug('item')

        self.assertIn(
            'item',
            str(cm.exception),
            "Exception message should contain errenous slug",
        )

        self.assertIn(
            'item_e, item_d, item_c',
            str(cm.exception),
            "Exception message should contain valid slugs",
        )

        self.assertIn(
            self.large_enum.name,
            str(cm.exception),
            "Exception message should contain enum name",
        )

    def test_get_choices(self):
        self.assertEqual(
            self.enum.get_choices(),
            [
                (Item(10, 'a', "Item A"), "Item A"),
                (Item(20, 'b', "Item B"), "Item B"),
            ],
        )

    def test_to_python(self):
        self.assertEqual(self.enum.to_python(''), None)
        self.assertEqual(self.enum.to_python(None), None)

        self.assertEqual(self.enum.to_python(self.enum.A), self.enum.A)

        self.assertEqual(self.enum.to_python(10), self.enum.A)
        self.assertEqual(self.enum.to_python(10), self.enum.A)

        self.assertEqual(self.enum.to_python('b'), self.enum.B)

        with self.assertRaises(ValueError):
            self.enum.to_python(999)

        with self.assertRaises(ValueError):
            self.enum.to_python('not_a_slug')

    def test_repr(self):
        self.assertEqual(
            repr(self.enum),
            "<FooEnum: [%r, %r]>" % (self.enum.A, self.enum.B),
        )


class FieldTests(DjangoTestCase):
    def assertCreated(self, num=1):
        self.assertEqual(TestModel.objects.count(), num)

    def test_model_instantiate(self):
        TestModel(
            test_field=TestModelEnum.A,
            test_field_no_default=TestModelEnum.B,
        )

    def test_model_creation(self):
        TestModel.objects.create(
            test_field=TestModelEnum.A,
            test_field_no_default=TestModelEnum.B,
        )

        self.assertCreated()

    def test_model_instantiate_using_defulat(self):
        TestModel(
            test_field_no_default=TestModelEnum.B,
        )

    def test_model_creation_using_defulat(self):
        TestModel.objects.create(
            test_field_no_default=TestModelEnum.B,
        )

        self.assertCreated()

    def test_model_instantiate_without_default(self):
        TestModel(
            test_field=TestModelEnum.A,
        )

    def test_model_creation_without_default(self):
        with self.assertRaises(IntegrityError):
            TestModel.objects.create(
                test_field=TestModelEnum.A,
            )

    def test_field_default(self):
        model = TestModel.objects.create(test_field_no_default=TestModelEnum.B)
        self.assertEqual(model.test_field, TestModelEnum.A)

    def test_field_from_slug(self):
        model = TestModel.objects.create(test_field_no_default='a')
        self.assertCreated()
        self.assertEqual(model.test_field_no_default, TestModelEnum.A)

    def test_field_from_value(self):
        model = TestModel.objects.create(test_field_no_default=20)
        self.assertCreated()
        self.assertEqual(model.test_field_no_default, TestModelEnum.B)

    def test_field_converts_to_python(self):
        model1 = TestModel(test_field_no_default='a')
        self.assertEqual(model1.test_field_no_default, TestModelEnum.A)

        model2 = TestModel(test_field_no_default=20)
        self.assertEqual(model2.test_field_no_default, TestModelEnum.B)

    def test_query(self):
        m1 = TestModel.objects.create(test_field_no_default=TestModelEnum.A)
        TestModel.objects.create(test_field_no_default=TestModelEnum.B)

        self.assertEqual(TestModel.objects.count(), 2)
        self.assertEqual(
            list(TestModel.objects.filter(
                test_field_no_default=TestModelEnum.A,
            )),
            [m1],
        )
        self.assertEqual(
            list(TestModel.objects.filter(
                test_field_no_default='a',
            )),
            [m1],
        )
        self.assertEqual(
            list(TestModel.objects.filter(
                test_field_no_default=10,
            )),
            [m1],
        )

    def test_null_field(self):
        TestModelNull.objects.create(test_field_null=None)

    def test_field_lookup(self):
        TestModelNull.objects.create(test_field_null=None)
        m2 = TestModelNull.objects.create(test_field_null=TestModelEnum.A)

        query = TestModelNull.objects.filter(
            test_field_null__in=(TestModelEnum.A, TestModelEnum.B),
        )

        self.assertEqual(list(query), [m2])

    def test_field_lookup_in_slugs(self):
        TestModelNull.objects.create(test_field_null=None)
        m2 = TestModelNull.objects.create(test_field_null=TestModelEnum.A)

        query = TestModelNull.objects.filter(test_field_null__in=('a', 'b'))

        self.assertEqual(list(query), [m2])

    def test_field_lookup_in_values(self):
        TestModelNull.objects.create(test_field_null=None)
        m2 = TestModelNull.objects.create(test_field_null=TestModelEnum.A)

        query = TestModelNull.objects.filter(test_field_null__in=(10, 20))

        self.assertEqual(list(query), [m2])

    def test_field_lookup_in_non_existent_slug_fails(self):
        with self.assertRaises(ValueError):
            TestModel.objects.filter(test_field__in=('not_a_slug',))

    def test_field_lookup_in_non_existent_value_fails(self):
        with self.assertRaises(ValueError):
            TestModel.objects.filter(test_field__in=(999,))

    def test_isnull(self):
        m1 = TestModelNull.objects.create(test_field_null=None)
        TestModelNull.objects.create(test_field_null=TestModelEnum.A)

        query = TestModelNull.objects.filter(test_field_null__isnull=True)

        self.assertEqual(list(query), [m1])

    def test_unsupported_lookup(self):
        if django.VERSION < (1, 10):
            # This feature is only supported pre-Django 1.10.
            with self.assertRaises(TypeError):
                TestModel.objects.filter(test_field__icontains=('blah',))


class TemplateTests(DjangoTestCase):
    def test_renders_template(self):
        kwargs = {'request': HttpRequest()}
        if django.VERSION < (1, 10):
            kwargs = {'context_instance': RequestContext(HttpRequest())}

        self.assertEqual(
            render_to_string('test.html', {}, **kwargs),
            "Item A, Item B\n",
        )

    def test_fails_loudly_for_invalid_app(self):
        kwargs = {'request': HttpRequest()}
        if django.VERSION < (1, 10):
            kwargs = {'context_instance': RequestContext(HttpRequest())}

        with self.assertRaises(TemplateErrorException):
            render_to_string('invalid.html', {}, **kwargs)


class UtilsTests(unittest.TestCase):
    def test_get_enum_or_404_valid(self):
        self.assertEqual(
            get_enum_or_404(TestModelEnum, 'a'),
            TestModelEnum.A,
        )

    def test_get_enum_or_404_invalid(self):
        with self.assertRaises(Http404):
            get_enum_or_404(TestModelEnum, 'not_a_slug')


class MigrationUnitTests(DjangoTestCase):
    def assertDeconstruct(self, model_class, field, exp_args, exp_kwargs):
        model = model_class()
        name, path, args, kwargs = model._meta.get_field(field).deconstruct()
        self.assertEqual(name, field)
        self.assertEqual(path, 'django.db.models.fields.IntegerField')
        self.assertEqual(args, exp_args)
        self.assertEqual(kwargs, exp_kwargs)

    def test_deconstruct(self):
        self.assertDeconstruct(TestModel, 'test_field', [], {'default': 10})

    def test_deconstruct_no_default(self):
        self.assertDeconstruct(TestModel, 'test_field_no_default', [], {})

    def test_deconstruct_null(self):
        self.assertDeconstruct(
            TestModelNull,
            'test_field_null',
            [],
            {'null': True},
        )

    def test_deconstruct_callable_default(self):
        self.assertDeconstruct(
            TestModelRandomDefault,
            'test_field',
            [],
            {'default': 10},
        )

    def test_field_clone(self):
        model = TestModel()
        field = model._meta.get_field('test_field_no_default')
        clone = field.clone()

        self.assertTrue(isinstance(clone, models.IntegerField))
        self.assertEqual(clone.default, NOT_PROVIDED)

    def test_field_clone_with_default(self):
        model = TestModel()
        field = model._meta.get_field('test_field')
        clone = field.clone()

        self.assertTrue(isinstance(clone, models.IntegerField))
        self.assertEqual(clone.default, 10)


class SerialisationTests(DjangoTestCase):
    def test_serialisation(self):
        m_in = TestModel.objects.create(test_field_no_default=TestModelEnum.B)

        data = serializers.serialize('xml', TestModel.objects.all())
        objects = serializers.deserialize('xml', data)

        m_out = next(objects).object

        self.assertEqual(m_in.pk, m_out.pk)
        self.assertEqual(m_in.test_field, m_out.test_field)
        self.assertEqual(
            m_in.test_field_no_default,
            m_out.test_field_no_default,
        )



SECRET_KEY = 'fake-key'
INSTALLED_APPS = [
    'tests',
    'tests.app',
]
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    },
}
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django_enumfield.context_processors.enumfield_context',
            ],
        },
    },
]
SILENCED_SYSTEM_CHECKS = [
    '1_7.W001',
]
ROOT_URLCONF = 'tests.urls'



import six
import functools

class ItemMeta(type):
    def __new__(mcs, name, bases, attrs):
        cls = super(ItemMeta, mcs).__new__(mcs, name, bases, attrs)

        try:
            item = cls(attrs['value'], name, attrs.get('display'))
        except KeyError:
            pass
        else:
            cls.__enum__.add_item(item)

        return cls

@functools.total_ordering
class Item(six.with_metaclass(ItemMeta, object)):
    def __init__(self, value, slug, display=None):
        if not isinstance(value, int):
            raise TypeError("item value should be an int, not %r" \
                % type(value))

        if not isinstance(slug, str):
            raise TypeError("item slug should be a str, not %r" % type(slug))

        if display is not None and not isinstance(display, six.string_types):
            raise TypeError("item display name should be a basestring, not %r" \
                % type(display))

        self.value = value
        self.slug = slug
        self.display = display if display is not None else slug.capitalize()

    def __str__(self):
        return self.slug

    def __repr__(self):
        return u"<enum.Item: %d %s %r>" % (self.value, self.slug, self.display)

    def __hash__(self):
        return self.value

    def __eq__(self, other):
        if isinstance(other, Item):
            return self.value == other.value

        if isinstance(other, six.integer_types + six.string_types):
            try:
                return self.value == int(other)
            except ValueError:
                return six.text_type(self.slug) == six.text_type(other)

        return False

    def __ne__(self, other):
        return not self.__eq__(other)

    def __lt__(self, other):
        if not isinstance(other, Item):
            return NotImplemented

        return self.value < other.value



import six

from django import VERSION
from django.db import models

if VERSION < (1, 10):
    metaclass = models.SubfieldBase
else:
    metaclass = type

class EnumField(six.with_metaclass(metaclass, models.Field)):
    def __init__(self, enum, *args, **kwargs):
        self.enum = enum

        kwargs.setdefault('choices', enum.get_choices())

        super(EnumField, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        return 'IntegerField'

    def to_python(self, value):
        return self.enum.to_python(value)

    def from_db_value(self, value, expression, connection, context):
        return self.to_python(value)

    def get_prep_value(self, value):
        python_value = self.to_python(value)

        if python_value is None:
            return None

        return python_value.value

    def get_prep_lookup(self, lookup_type, value):
        def prepare(value):
            x = self.to_python(value)

            return self.get_prep_value(x)

        if lookup_type in ('exact', 'lt', 'lte', 'gt', 'gte'):
            return prepare(value)
        elif lookup_type == 'in':
            return [prepare(v) for v in value]
        elif lookup_type == 'isnull':
            return value

        raise TypeError("Lookup type %r not supported." % lookup_type)

    def value_to_string(self, obj):
        item = self.value_from_object(obj)
        return str(item.value)

    def clone(self):
        _, _, args, kwargs = self.deconstruct()
        return models.IntegerField(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(EnumField, self).deconstruct()

        # If there is a callable default, override it and set the first item
        # from the enum. This is to stop randomised defaults causing unstable
        # migrations, as deconstruct is called every time makemigrations is run
        default = kwargs.get('default')
        if default and callable(default):
            kwargs['default'] = self.enum[0]

        try:
            kwargs['default'] = kwargs['default'].value
        except (KeyError, AttributeError):
            # No default or not an Item instance
            pass

        # We don't want to serialise this for migrations.
        del kwargs['choices']

        return name, 'django.db.models.fields.IntegerField', args, kwargs



import inspect

from django.apps import apps
from django.utils.lru_cache import lru_cache

from .enum import Enum
from .utils import TemplateErrorDict

def enumfield_context(request):
    return {'enums': get_enums()}

@lru_cache()
def get_enums():
    result = TemplateErrorDict("Unknown app name %s")

    for app_config in apps.get_app_configs():
        module = getattr(__import__(app_config.name, {}, {}, ('enums',)), 'enums', None)

        if module is None:
            continue

        for _, x in inspect.getmembers(module):
            if not isinstance(x, Enum):
                continue

            app_name = app_config.name.split('.')[-1]

            result.setdefault(
                app_name,
                TemplateErrorDict("Unknown enum %%r in %r app" % app_name),
            )[x.name] = x

    return result



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name='django-enumfield-ng',
    description="Type-safe, efficient & database-agnostic enumeration field "
        'for Django.',
    version='1.2.0',

    url="https://chris-lamb.co.uk/projects/django-enumfield",
    author="Chris Lamb",
    author_email='chris@chris-lamb.co.uk',
    license='BSD',

    packages=find_packages(),

    install_requires=(
        'Django>=1.8',
        'six',
    ),
)



from django.http import Http404

def get_enum_or_404(enum, slug):
    try:
        return enum.from_slug(slug)
    except ValueError:
        raise Http404()

class TemplateErrorDict(dict):
    """
    Like a regular dict but raises our own exception instead of ``KeyError`` to
    bypass Django's silent variable swallowing.
    """

    def __init__(self, template, *args, **kwargs):
        self.template = template

        super(TemplateErrorDict, self).__init__(*args, **kwargs)

    def __getitem__(self, key):
        if key not in self:
            raise TemplateErrorException(self.template % key)

        return super(TemplateErrorDict, self).__getitem__(key)

class TemplateErrorException(RuntimeError):
    silent_variable_failure = False



import random

from django_enumfield import EnumField

from django.db import models

from .enums import TestModelEnum


class TestModel(models.Model):
    test_field = EnumField(TestModelEnum, default=TestModelEnum.A)
    test_field_no_default = EnumField(TestModelEnum)


class TestModelNull(models.Model):
    test_field_null = EnumField(TestModelEnum, null=True)


def random_default():
    return random.choice(TestModelEnum)


class TestModelRandomDefault(models.Model):
    test_field = EnumField(TestModelEnum, default=random_default)



import difflib

import six

from .item import Item


class NoSuchSlugValueError(ValueError):
    def __init__(self, slug, enum):
        self.slug = slug
        self.enum = enum
        super(ValueError, self).__init__()

    def _message(self):
        valid_slugs = [x.slug for x in self.enum]

        if len(valid_slugs) <= 3:
            corrective_message = "Slugs: %s" % ', '.join(valid_slugs)
        else:
            best_matches = difflib.get_close_matches(self.slug, valid_slugs)
            corrective_message = "Close matches: %s" % ', '.join(best_matches)

        return "%r is not a valid slug for enum %s; %s" % (
            self.slug,
            self.enum.name,
            corrective_message,
        )

    __str__ = _message

    def repr(self):
        return "%s(%r)" % (
            type(self).__name__,
            self._message(),
        )


class Enum(list):
    def __init__(self, name, *items):
        self.name = name

        super(Enum, self).__init__()

        for x in items:
            self.add_item(x)

    def __repr__(self):
        return "<%s: %s>" % (self.name, list(self))

    def add_item(self, item):
        for name, fn, value in (
            ('value', self.from_value, item.value),
            ('slug', self.from_slug, item.slug),
        ):
            try:
                fn(value)
            except ValueError:
                pass
            else:
                raise ValueError("Duplicate item %s: %r" % (name, value))

        setattr(self, item.slug.upper(), item)

        self.append(item)

    def from_value(self, value):
        if not isinstance(value, int):
            # Allow values that convert to int, as we might be deserialising an
            # int value. Raises ValueError and falls through to_python
            # accordingly if this is not actually an int.
            value = int(value)

        try:
            return {x.value: x for x in self}[value]
        except KeyError:
            raise ValueError("%r is not a valid value for enum %s" % (value, self.name))

    def from_slug(self, slug):
        if not isinstance(slug, six.string_types):
            raise TypeError("item slug should be a str, not %r" % type(slug))

        try:
            return {x.slug.lower(): x for x in self}[slug.lower()]
        except KeyError:
            raise NoSuchSlugValueError(slug=slug, enum=self)

    def get_choices(self):
        return [(x, x.display) for x in self]

    def to_python(self, value):
        if value in (None, '', u''):
            return None

        if isinstance(value, Item):
            return value

        try:
            return self.from_value(value)
        except ValueError:
            pass

        try:
            return self.from_slug(value)
        except (ValueError, TypeError):
            pass

        raise ValueError("%r is not a valid slug or value for enum %s" % (value, self.name))
